/*
	Open device files and initialize variables
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <pthread.h>

#include "func_extern.h"
#include "control.h"
//#include "irs_ts_ws.h"
#include "motor.h"
#include "uss.h"

extern FILE *fp,*uss_fp,*state_fp,*step_fp,*obst_fp,*eod_fp;
extern enum states {STRAIGHT,TIMER_STRAIGHT,MOTOR_RESTART,R_RESTORE,L_RESTORE,R_TURN,R_RETURN,L_TURN,L_RETURN,KEY,DB,ROTARY,KOBEYA,GOAL};
extern enum states state,state_tmp,state_tmp_keep;
extern enum {ENTRY,DO,EXIT} eod;
extern int obst,obst_keep;
extern int obst_log[10];
extern int step,step_keep;
extern time_t start_time;

int initialize(void){
	int ret,i=0;
	printf("Initialization¥n");

        //スレッドの設定

	pthread_attr_init (&attr_motor1);
	sched_param.sched_priority = 23;
	pthread_attr_setschedpolicy(&attr_motor1, SCHED_RR);
	pthread_attr_setschedparam (&attr_motor1, &sched_param);

	pthread_attr_init (&attr_gyro);
	sched_param.sched_priority = 24;
	pthread_attr_setschedpolicy(&attr_gyro, SCHED_RR);
	pthread_attr_setschedparam (&attr_gyro, &sched_param);


	pthread_attr_init (&attr_irs_ts_ws);
	sched_param.sched_priority = 20;
	pthread_attr_setschedpolicy(&attr_irs_ts_ws, SCHED_RR);
	pthread_attr_setschedparam(&attr_irs_ts_ws, &sched_param);

	pthread_attr_init (&attr_uss);
	sched_param.sched_priority = 21;
	pthread_attr_setschedpolicy(&attr_uss, SCHED_RR);
	pthread_attr_setschedparam (&uss_task, &sched_param);

	pthread_attr_init (&attr_timer);
	sched_param.sched_priority = 22;
	pthread_attr_setschedpolicy(&attr_timer, SCHED_RR);
	pthread_attr_setschedparam (&timer_task, &sched_param);

	printf("opening device...¥n");
	fd1 = open(M_PORT_L, O_RDWR);
	if(fd1 < 0){
		if(errno == 3) printf("no such file¥n");
		else if(errno == 13) printf("permission¥n");
		else printf("other¥n");
		fprintf(stderr, "fd1 open error¥n");
		exit(-1);
	}
	fd2 = open(M_PORT_R, O_RDWR);
	if(fd2 < 0){
		fprintf(stderr, "fd2 open error¥n");
		close(fd1);
		exit(-1);
	}
	if ((fd4 = open("/dev/irs_ts_ws", O_RDONLY)) < 0) {
                fprintf(stderr, "Error opening /dev/irs_ts¥n");
                exit(1);
        }
	printf("end¥n");

	fp=fopen("../output/out.txt","w");
	fprintf(fp,"/**********initialize**********/\n");
	fclose(fp);

	state_fp=fopen("../output/state.txt","r");
	if(state_fp == NULL){
		printf("state file open error\n");
	}else{
		while(( ret = fscanf( state_fp,"%d,%d,%d",&state,&state_tmp,&state_tmp_keep ))!= EOF ){ }
		fclose(state_fp);
	}

	step_fp=fopen("../output/step.txt","r");
	if(step_fp == NULL){
		printf("step file open error\n");
	}else{
		while(( ret = fscanf( step_fp,"%d,%d",&step,&step_keep))!= EOF ){ }
		fclose(step_fp);
	}

	i=0;
	obst_fp=fopen("../output/obst.txt","r");
	if(obst_fp == NULL){
		printf("obst file open error\n");
	}else{
		while(( ret = fscanf( obst_fp,"%d,%d",&obst,&obst_keep))!= EOF ){ }
		fclose(obst_fp);
		obst_log[i]=obst;
		i++;
	}

	eod_fp=fopen("../output/eod.txt","r");
	if(eod_fp == NULL){
		printf("eod file open error\n");
	}else{
		while(( ret = fscanf( eod_fp,"%d",&eod ))!= EOF ){ }
		fclose(eod_fp);
	}

	fp=fopen("../output/out.txt","a");
	uss_fp=fopen("../output/uss.txt","w");
	state_fp=fopen("../output/state.txt","w");
	step_fp=fopen("../output/step.txt","w");
	obst_fp=fopen("../output/obst.txt","w");
	eod_fp=fopen("../output/eod.txt","w");

	flag_reset();

	if(i==0){
		pthread_create(&irs_ts_ws_task, &attr_irs_ts_ws,irs_ts_ws_thread,NULL);
		pthread_create(&motor_task1, &attr_motor1,introduction1,NULL);
		pthread_join(motor_task1, NULL);
		pthread_join(irs_ts_ws_task,NULL);
	}else{
		time(&start_time);
		pthread_create(&timer_task, &attr_timer,timer_thread,NULL);
		pthread_create(&irs_ts_ws_task, &attr_irs_ts_ws,irs_ts_ws_thread,NULL);
		pthread_create(&motor_task1, &attr_motor1,introduction2,NULL);
		pthread_join(motor_task1, NULL);
		pthread_join(irs_ts_ws_task,NULL);
		pthread_join(timer_task, NULL);
	}
	return 0;
}
